// react:
import { 
// hooks:
useRef, useEffect, } from 'react';
// cssfn:
import { 
// writes css in javascript:
rule, states, style, vars, cssVars, } from '@cssfn/core'; // writes css in javascript
// reusable-ui utilities:
import { 
// hooks:
useTriggerRender, useEvent, } from '@reusable-ui/hooks'; // react helper hooks
// reusable-ui features:
import { 
// hooks:
usesAnimation, } from '@reusable-ui/animation'; // animation stuff of UI
const [excitableVars] = cssVars({ prefix: 'ex', minify: false }); // shared variables: ensures the server-side & client-side have the same generated css variable names
{
    const { animationRegistry: { registerFilter, registerAnim } } = usesAnimation();
    registerFilter(excitableVars.filter);
    registerAnim(excitableVars.anim);
}
// parent is     `.excited` -or-  current is     `.excited`:
export const ifExcited = (styles) => rule(':is(.excited&, &.excited)', styles); // specificityWeight = 1 + (parent's specificityWeight)
// parent is not `.excited` -and- current is not `.excited`:
export const ifNotExcited = (styles) => rule(':where(&):not(:is(.excited&, &.excited))', styles); // specificityWeight = 1 + (parent's specificityWeight)
/**
 * Adds a capability of UI to highlight itself to attract user's attention.
 * @param config  A configuration of `excitableRule`.
 * @returns An `ExcitableStuff` represents an excitable state.
 */
export const usesExcitable = (config) => {
    return {
        excitableRule: () => style({
            // animation states:
            ...states([
                ifExcited({
                    ...vars({
                        [excitableVars.filter]: config?.filterExcite,
                        [excitableVars.anim]: config?.animExcite,
                    }),
                }),
            ]),
        }),
        excitableVars,
    };
};
export const useToggleExcitable = (props) => {
    // props:
    const { 
    // states:
    excited, onExcitedChange, } = props;
    /*
     * the state is excited/normal based on [controllable excited]
     */
    const excitedFn = (excited /*controllable*/ ?? false);
    // states:
    const isUnloaded = useRef(false);
    useEffect(() => {
        // setups:
        isUnloaded.current = false;
        // cleanups:
        return () => {
            isUnloaded.current = true;
        };
    }, []);
    // local storages without causing to (re)render, we need to manual control the (re)render event:
    /**
     * `true`  => has excited
     * `false` => has normal
     * `null`  => need to restart
     */
    const isExcited = useRef(excitedFn);
    // manually controls the (re)render event:
    const [triggerRender] = useTriggerRender();
    if ((isExcited.current !== null) && (isExcited.current !== excitedFn)) { // change detected => apply the change & start animating
        isExcited.current = excitedFn; // remember the last change
        triggerRender(); // need to apply the animation
    } // if
    // dom effects:
    useEffect(() => {
        // conditions:
        if (isExcited.current !== null)
            return; // only process `null` (need to restart)
        // setups:
        // need to *briefly* apply the *un-animated* before continue to *re-animated*:
        const cancelRequest = requestAnimationFrame(() => {
            // conditions:
            if (isUnloaded.current)
                return;
            // actions:
            isExcited.current = true; // restart
            triggerRender(); // need to restart the animation
        });
        // cleanups:
        return () => {
            cancelAnimationFrame(cancelRequest);
        };
    }); // no dependency-list, runs every re-render
    // handlers:
    const handleAnimationStart = useEvent((event) => {
        // conditions:
        if (event.target !== event.currentTarget)
            return; // ignores bubbling
        if (!/((?<![a-z])(excite)|(?<=[a-z])(Excite))(?![a-z])/.test(event.animationName))
            return; // ignores animation other than excite[Foo] or booExcite[Foo]
        // mark the animation has started:
        isExcited.current = true;
    });
    const handleAnimationEnd = useEvent((event) => {
        // conditions:
        if (event.target !== event.currentTarget)
            return; // ignores bubbling
        if (!/((?<![a-z])(excite)|(?<=[a-z])(Excite))(?![a-z])/.test(event.animationName))
            return; // ignores animation other than excite[Foo] or booExcite[Foo]
        // mark the animation has stopped:
        if (!excitedFn) {
            isExcited.current = false; // mark the animation has stopped
        }
        else {
            isExcited.current = null; // mark the animation has stopped but need to restart
            triggerRender(); // need to restart the animation
        } // if
        // request to stop:
        setTimeout(() => {
            onExcitedChange?.({ excited: false });
        }, 0); // runs the 'onExcitedChange' event *next after* current event completed
    });
    const handleAnimationCancel = handleAnimationEnd;
    return {
        excited: excitedFn,
        class: isExcited.current ? 'excited' : null,
        handleAnimationStart,
        handleAnimationEnd,
        handleAnimationCancel, // animation-cancel handler
    };
};
//#endregion excitable
